import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'parsed_route.dart';
import 'common.dart';

class IRouteParser extends RouteInformationParser<ParsedRoute> {
  final Map<String, RouteBuilder> routes;
  final RouteGuard<ParsedRoute>? guard;
  final ParsedRoute initialRoute;

  IRouteParser({
    String initialRoute = '/',
    required this.routes,
    this.guard,
  })  : initialRoute = ParsedRoute(initialRoute),
        assert(routes.containsKey(initialRoute));

  @override
  Future<ParsedRoute> parseRouteInformation(
    RouteInformation routeInformation,
  ) async {
    final path = routeInformation.location!;
    var parsedRoute = ParsedRoute(path);

    var guard = this.guard;
    if (guard != null) {
      return guard(parsedRoute);
    }

    return SynchronousFuture(parsedRoute);
  }

  @override
  RouteInformation restoreRouteInformation(ParsedRoute configuration) =>
      RouteInformation(location: configuration.path);
}
