class ParsedRoute {
  final String path;
  late final String pattern;

  late final Map<String, dynamic> parameters;

  static const dynamicParams = {':ipage'};

  static String getPattern(String url) {
    String pattern =
        url.replaceAllMapped(RegExp('(:[sid][^=]+)(=[^/]+)'), (match) {
      if (dynamicParams.contains(match.group(1))) return '';
      return '${match.group(1)}';
    });
    return pattern;
  }

  static Map<String, dynamic> getParam(String url) {
    var regExp = RegExp('(:[sid][^=]+)(=[^/^:]+)');
    var result = regExp.allMatches(url);
    Map<String, dynamic> args = <String, dynamic>{};
    for (RegExpMatch match in result) {
      var argName = match.group(1)!.substring(2);
      var argType = match.group(1)![1];
      var argValue = match.group(2)!.substring(1); //去掉等号
      dynamic value;
      if (argType == 'i') value = int.tryParse(argValue);
      if (argType == 'd') value = double.tryParse(argValue);
      if (argType == 's') value = argValue;
      args[argName] = value;
    }
    return args;
  }

  ParsedRoute(this.path)
      : pattern = getPattern(path),
        parameters = getParam(path);

  @override
  bool operator ==(Object other) => other is ParsedRoute && other.path == path;

  @override
  int get hashCode => path.hashCode;

  @override
  String toString() => '<ParsedRoute '
      'pattern: $pattern '
      'path: $path '
      'parameters: $parameters ';
}
